require_relative '../../spec_helper'

ruby_version_is ""..."3.1" do
  require 'prime'

  describe "Prime.int_from_prime_division" do
    it "returns the product of the given factorization" do
      Prime.int_from_prime_division([[2,3], [3,3], [5,3], [7,3], [11,3], [13,3], [17,3]]).
        should == 2**3 * 3**3 * 5**3 * 7**3 * 11**3 * 13**3 * 17**3
    end

    it "returns 1 for an empty factorization" do
      Prime.int_from_prime_division([]).should == 1
    end
  end
end
