require_relative '../../../spec_helper'
require 'uri'

describe "URI::Generic#scheme" do
  it "needs to be reviewed for spec completeness"
end

describe "URI::Generic#scheme=" do
  it "needs to be reviewed for spec completeness"
end
