require_relative '../../../spec_helper'

ruby_version_is ""..."3.1" do
  require_relative 'spec_helper'

  describe "Net::FTP#set_socket" do
    # TODO: I won't spec this method, as it is not used
    #       anywhere and it should be private anyway.
    it "needs to be reviewed for spec completeness"
  end
end
